#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = require("commander");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const update_notifier_1 = __importDefault(require("update-notifier"));
const build_1 = __importDefault(require("../commands/build"));
const convert_1 = __importDefault(require("../commands/convert"));
const create_1 = __importDefault(require("../commands/create"));
const screenshot_1 = __importDefault(require("../commands/screenshot"));
const serve_1 = __importDefault(require("../commands/serve"));
const tree_1 = __importDefault(require("../commands/tree"));
const program = new commander_1.Command();
const pkg = JSON.parse(fs_1.default.readFileSync(path_1.default.join(__dirname, '..', '..', 'package.json'), 'utf-8'));
const appVersion = pkg.version;
program.version(appVersion);
(0, update_notifier_1.default)({ pkg }).notify({
    isGlobal: true,
    message: `Update available ${chalk_1.default.dim('{currentVersion}')} ${chalk_1.default.reset('→')} ${chalk_1.default.green('{latestVersion}')}
  Run ${chalk_1.default.cyan('{updateCommand}')} to update
  Changes: ${chalk_1.default.blue('https://bit.ly/p5-server-changelog')}`,
});
program
    .command('create')
    .description('Create a new p5.js sketch')
    .alias('c')
    .alias('generate')
    .alias('g')
    .argument('[FILE]', 'the filename of the sketch', 'sketch.js')
    .option('--type <TYPE>', 'place the new files in a folder')
    .option('--force', 'force overwrite of existing files')
    .option('--options [OPTIONS]', 'comma-separated list of template options')
    .option('-t, --title [TITLE]', 'sketch title')
    .action(create_1.default);
program
    .command('build')
    .description('Create a static site that presents the sketches')
    .alias('b')
    .argument('[SOURCE]', 'the source directory', '.')
    .option('-o, --output <OUTPUT>', 'the output directory', 'build')
    .option('--open', 'Open the index file in a browser')
    .option('--options [OPTIONS]', 'comma-separated list of template options')
    .option('-t, --theme <FILE>', 'template file', 'split')
    .option('-v, --verbose', 'verbose output')
    .option('--dry-run', 'dry run')
    .action(build_1.default);
program
    .command('convert')
    .description('Convert an HTML sketch to JavaScript-only or vice versa')
    .argument('file')
    .option('--to <type>', 'output type: html or javascript')
    .action(convert_1.default);
program.command('libraries', 'List the libraries', {
    hidden: true,
    executableFile: 'p5-analyze',
});
program
    .command('screenshot')
    .argument('SKETCH_FILE')
    .option('-o, --output <OUTPUT>', 'the output file')
    .option('--browser <NAME>', 'safari | chrome | firefox | edge')
    .option('--canvas-size <DIMENSIONS>', 'canvas dimensions e.g. 100 or 100,200')
    .option('--frame-count <NUMBER>', 'the number of frames saved', '1')
    .option('--pixel-density <NUMBER>', 'e.g. 2, 0.5, or 1/2')
    .option('--skip-frames <COUNT>', 'omit the first COUNT frames')
    .action(screenshot_1.default);
program
    .command('serve')
    .argument('[DIRECTORY...]', 'the directory to serve', '.')
    .description('Start the web server')
    .alias('server')
    .alias('r')
    .alias('run')
    .alias('r')
    .option('-o, --open', 'Open the page in a browser')
    .option('-p, --port [PORT]', 'HTTP port to listen on', '3000')
    .option('-t, --theme [FILE]', 'template file')
    .option('--browser <NAME>', 'safari | chrome | firefox | edge (implies --open)')
    .option('--split', 'Use the split (directory + sketch) template')
    .option('--console [FORMAT]', 'Relay console messages and errors to sketch in the server console')
    .action(serve_1.default);
program
    .command('tree')
    .description('Print the tree structure of a directory and its sketches')
    .argument('[DIRECTORY]', 'directory', '.')
    .option('-L, --level <LEVEL>', 'Descend only level directories deep.')
    .option('--descriptions', 'Print descriptions of sketches')
    .action(tree_1.default);
program.parse(process.argv);
