"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const p5_analysis_1 = require("p5-analysis");
const path_1 = __importDefault(require("path"));
const printTree_1 = require("../printTree");
async function tree(file, options) {
    const depth = options.level || Infinity;
    const iter = sketchTreeIter(file, { depth, printDescriptions: options.descriptions });
    await (0, printTree_1.printTree)(iter);
}
exports.default = tree;
function sketchTreeIter(file, { depth, printDescriptions }) {
    return visit(file, depth);
    async function* visit(file, depth) {
        if (fs_1.default.statSync(file).isDirectory()) {
            yield* visitDir(file, depth - 1);
        }
        else if (await p5_analysis_1.Sketch.isSketchFile(file)) {
            const sketch = await p5_analysis_1.Sketch.fromFile(file);
            yield* visitSketch(sketch, depth - 1);
        }
        else {
            yield path_1.default.basename(file);
        }
    }
    async function* visitDir(dir, depth) {
        const { sketches, unassociatedFiles } = await p5_analysis_1.Sketch.analyzeDirectory(dir);
        yield* ['📁' + path_1.default.basename(dir), printTree_1.indentSymbol];
        if (depth >= 0) {
            for (const sketch of sketches.sort((a, b) => a.name.localeCompare(b.name))) {
                yield* visitSketch(sketch, depth - 1);
            }
            const files = [...unassociatedFiles]
                .map(f => path_1.default.join(dir, f))
                .sort((a, b) => a.localeCompare(b))
                .sort((a, b) => Number(fs_1.default.statSync(b).isDirectory()) - Number(fs_1.default.statSync(a).isDirectory()));
            for (const file of files) {
                yield* visit(file, depth - 1);
            }
        }
        yield printTree_1.dedentSymbol;
    }
    async function* visitSketch(sketch, depth) {
        const mainFile = sketch.files.length === 1 ? ` (${sketch.mainFile})` : '';
        const description = printDescriptions && sketch.description
            ? ` - ${sketch.description.replace(/\s*\n\s*/g, ' ')}`
            : '';
        yield* ['🎨' + `${sketch.name}${mainFile}${description}`, printTree_1.indentSymbol];
        if (depth >= 0 && sketch.files.length > 1) {
            yield* sketch.files;
        }
        yield printTree_1.dedentSymbol;
    }
}
