"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.printTree = exports.dedentSymbol = exports.indentSymbol = void 0;
exports.indentSymbol = Symbol('indent');
exports.dedentSymbol = Symbol('dedent');
async function printTree(iter) {
    const prefixStack = [];
    let prefix = '';
    for await (const { item, index, isFirst, isLast } of addTreeProperties(iter)) {
        if (isFirst && index > 0) {
            prefixStack.push(prefix);
            prefix = prefix.replace(/├/g, '│').replace(/─/g, ' ') + '├── ';
        }
        if (isLast) {
            prefix = prefix.replace('├', '└');
        }
        console.log(prefix + item);
        if (isLast) {
            prefix = prefixStack.pop();
        }
    }
}
exports.printTree = printTree;
async function* addTreeProperties(iter) {
    let index = 0;
    for await (const [prev, item, next] of asyncIterateWindows(dropEmptyGroups(iter), 3)) {
        switch (item) {
            case exports.indentSymbol:
            case exports.dedentSymbol:
                break;
            default:
                if (item !== undefined) {
                    const isFirst = !prev || prev === exports.indentSymbol;
                    const isLast = !next || next === exports.dedentSymbol;
                    yield { item, isFirst, isLast, index: index++ };
                }
        }
    }
}
async function* dropEmptyGroups(iter) {
    let indentation = 0;
    for await (const item of iter) {
        switch (item) {
            case exports.indentSymbol:
                indentation++;
                break;
            case exports.dedentSymbol:
                if (indentation > 0) {
                    indentation--;
                }
                else {
                    yield item;
                }
                break;
            default:
                while (indentation > 0) {
                    indentation--;
                    yield exports.indentSymbol;
                }
                yield item;
        }
    }
}
async function* asyncIterateWindows(iter, width) {
    const window = new Array(width);
    for await (const x of iter) {
        window.shift();
        window.push(x);
        yield window;
    }
    for (let i = 0; i < width - 1; i++) {
        window.shift();
        window.push(undefined);
        yield window;
    }
}
