/// <reference types="node" />
import { EventEmitter } from 'stream';
import http = require('http');
export declare namespace Server {
    type Options = Partial<{
        port: number;
        scanPorts: boolean;
        root: string | null;
        mountPoints: MountPointOption[];
        relayConsoleMessages: boolean;
        liveServer: boolean;
        screenshot: Partial<{
            canvasDimensions: {
                width: number;
                height: number;
            };
            frameCount: number;
            imageType: 'png' | 'jpeg';
            pixelDensity: number;
            skipFrames: number;
            onFrameData: (data: {
                data: Buffer;
                frameNumber: number;
                imageType: string;
            }) => void | Promise<void>;
        }> | null;
        theme?: string;
    }>;
    type MountPointOption = string | {
        filePath: string;
        name?: string;
        urlPath?: string;
    };
}
declare type MountPoint = {
    filePath: string;
    urlPath: string;
    name?: string;
};
export declare class Server {
    server: http.Server | null;
    url?: string;
    mountPoints: MountPoint[];
    private readonly config;
    private liveReloadServer;
    private readonly browserScriptEmitter;
    readonly emitScriptEvent: (eventName: string | symbol, ...args: any[]) => boolean;
    readonly onScriptEvent: (eventName: string | symbol, listener: (...args: any[]) => void) => EventEmitter;
    constructor(options?: Partial<Server.Options>);
    static start(options?: Partial<Server.Options>): Promise<Server>;
    start(): Promise<this>;
    stop(): Promise<void>;
    filePathToUrl(filePath: string): string | null;
    urlPathToFilePath(urlPath: string): string | null;
    private static normalizeMountPoints;
}
export {};
