"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.closeSync = exports.listenSync = void 0;
function listenSync(app, port, timeout = 1000) {
    return new Promise((resolve, reject) => {
        const server = app.listen(port);
        server.on('error', onError);
        const timeoutTimer = setTimeout(() => {
            const address = server.address();
            removeListeners();
            if (address) {
                resolve(server);
            }
            else {
                reject(new Error('Failed to start server'));
            }
        }, timeout);
        const intervalTimer = setInterval(() => {
            const address = server.address();
            if (address) {
                removeListeners();
                resolve(server);
            }
        }, 50);
        function onError(e) {
            removeListeners();
            reject(e);
        }
        function removeListeners() {
            server.off('error', onError);
            clearTimeout(timeoutTimer);
            clearInterval(intervalTimer);
        }
    });
}
exports.listenSync = listenSync;
function closeSync(server, timeout = 1000) {
    return new Promise((resolve, reject) => {
        server.close();
        server.on('close', onClose);
        server.on('error', onError);
        const timeoutTimer = setTimeout(() => {
            removeListeners();
            onError(new Error('Failed to close server'));
        }, timeout);
        function onClose() {
            removeListeners();
            resolve();
        }
        function onError(e) {
            removeListeners();
            reject(e);
        }
        function removeListeners() {
            server.off('close', onClose);
            server.off('error', onError);
            clearTimeout(timeoutTimer);
        }
    });
}
exports.closeSync = closeSync;
