"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLiveReloadServer = exports.injectLiveReloadScript = exports.liveReloadTemplate = void 0;
const livereload_1 = __importDefault(require("livereload"));
const net_1 = __importDefault(require("net"));
const utils_1 = require("../utils");
exports.liveReloadTemplate = `
  document.write('<script src="http://' + (location.host || 'localhost').split(':')[0] +
  ':$(port)/livereload.js?snipver=1"></' + 'script>')`;
function injectLiveReloadScript(html, liveReloadServer) {
    if (!liveReloadServer)
        return html;
    const address = liveReloadServer.server.address();
    if (typeof address === 'string') {
        throw new Error(`liveReloadServer.address is a string ${address}; expected a WebSocket.AddressInfo`);
    }
    const liveReloadScript = exports.liveReloadTemplate.replace('$(port)', address.port.toString());
    return (0, utils_1.addScriptToHtmlHead)(html, { script: liveReloadScript });
}
exports.injectLiveReloadScript = injectLiveReloadScript;
async function createLiveReloadServer({ port = 35729, scanPorts = true, watchDirs = [], }) {
    const lastPort = port + 9;
    while (port && scanPorts && !(await isPortAvailable(port))) {
        if (++port > lastPort)
            port = 0;
    }
    const lrServer = livereload_1.default.createServer({ port });
    watchDirs.forEach(dir => lrServer.watch(dir));
    return lrServer;
}
exports.createLiveReloadServer = createLiveReloadServer;
async function isPortAvailable(port) {
    for (let i = 0; i < 3; i++) {
        if (!(await _isPortAvailable(port)))
            return false;
        await new Promise(resolve => setTimeout(resolve, 2));
    }
    return true;
}
function _isPortAvailable(port) {
    return new Promise(resolve => {
        const server = net_1.default.createServer();
        server.once('error', () => resolve(false));
        server.once('close', () => resolve(true));
        server.once('listening', () => new Promise(resolve => setTimeout(resolve, 5)).then(() => server.close()));
        server.listen(port);
    });
}
