"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSyntaxErrorJsReporter = exports.syntaxErrorTemplate = exports.sourceViewTemplate = exports.markdownToHtmlPage = exports.markedOptions = exports.templateDir = void 0;
const fs_1 = __importDefault(require("fs"));
const core_1 = __importDefault(require("highlight.js/lib/core"));
const css_1 = __importDefault(require("highlight.js/lib/languages/css"));
const javascript_1 = __importDefault(require("highlight.js/lib/languages/javascript"));
const plaintext_1 = __importDefault(require("highlight.js/lib/languages/plaintext"));
const shell_1 = __importDefault(require("highlight.js/lib/languages/shell"));
const marked_1 = __importDefault(require("marked"));
const nunjucks_1 = __importDefault(require("nunjucks"));
const path_1 = __importDefault(require("path"));
const pug_1 = __importDefault(require("pug"));
const terminalCodes_1 = require("../terminalCodes");
core_1.default.registerLanguage('css', css_1.default);
core_1.default.registerLanguage('javascript', javascript_1.default);
core_1.default.registerLanguage('plaintext', plaintext_1.default);
core_1.default.registerLanguage('shell', shell_1.default);
exports.templateDir = path_1.default.join(__dirname, './templates');
const markdownPageTemplate = pug_1.default.compileFile(path_1.default.join(exports.templateDir, 'markdown.pug'));
exports.markedOptions = {
    smartypants: true,
    highlight(code, lang) {
        const language = core_1.default.getLanguage(lang) ? lang : 'javascript';
        return core_1.default.highlight(code, { language }).value;
    }
};
function markdownToHtmlPage(data) {
    const markdown = (0, marked_1.default)(data, exports.markedOptions);
    const title = (data.match(/^# (.*)$/m) || [])[1] ?? '';
    return markdownPageTemplate({ markdown, title });
}
exports.markdownToHtmlPage = markdownToHtmlPage;
exports.sourceViewTemplate = pug_1.default.compileFile(path_1.default.join(exports.templateDir, 'source-view.pug'));
exports.syntaxErrorTemplate = pug_1.default.compileFile(path_1.default.join(exports.templateDir, 'syntax-error.pug'));
const jsTemplateEnv = new nunjucks_1.default.Environment(null, { autoescape: false });
jsTemplateEnv.addFilter('quote', JSON.stringify);
const syntaxErrorJsTemplate = fs_1.default.readFileSync(path_1.default.join(exports.templateDir, 'report-syntax-error.js.njk'), 'utf-8');
function createSyntaxErrorJsReporter(errs, filepath) {
    const [message, context] = errs[0].message.split('\n\n', 2);
    const errorHtml = (0, exports.syntaxErrorTemplate)({
        message,
        context: (0, terminalCodes_1.terminalCodesToHtml)(context, true).replace(/\n/g, '<br>')
    });
    return jsTemplateEnv.renderString(syntaxErrorJsTemplate, {
        fileName: path_1.default.basename(filepath),
        message: (0, terminalCodes_1.removeTerminalCodes)(errs[0].message),
        errorHtml
    });
}
exports.createSyntaxErrorJsReporter = createSyntaxErrorJsReporter;
