"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addScriptToHtmlHead = exports.stringToOptions = exports.pathIsInDirectory = exports.pathComponentsForBreadcrumbs = exports.openInBrowser = exports.escapeHTML = exports.die = void 0;
const chalk_1 = __importDefault(require("chalk"));
const node_html_parser_1 = require("node-html-parser");
const open_1 = __importDefault(require("open"));
const path_1 = __importDefault(require("path"));
function die(message) {
    console.error(chalk_1.default.red('Error:', message));
    process.exit(1);
}
exports.die = die;
function escapeHTML(str) {
    return str
        .replace(/&/g, '&amp;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;')
        .replace(/"/g, '&quot;')
        .replace(/'/g, '&#39;');
}
exports.escapeHTML = escapeHTML;
function openInBrowser(url, browser) {
    const appName = browser === 'safari'
        ? 'safari'
        : browser in open_1.default.apps
            ? browser
            : undefined;
    if (browser && !browser) {
        die(`Unknown browser: ${browser}`);
    }
    const openApps = { safari: 'safari', ...open_1.default.apps };
    const openOptions = appName ? { app: { name: openApps[appName] } } : {};
    (0, open_1.default)(url, openOptions);
}
exports.openInBrowser = openInBrowser;
function pathComponentsForBreadcrumbs(relDirPath) {
    return (relDirPath
        .replace(/\/$/, '')
        .split('/')
        .slice(1)
        .reduce((crumbs, name) => [
        ...crumbs,
        {
            name,
            path: (crumbs[crumbs.length - 1].path + '/').replace('//', '/') + name,
        },
    ], [{ name: 'Home', path: '/' }]));
}
exports.pathComponentsForBreadcrumbs = pathComponentsForBreadcrumbs;
function pathIsInDirectory(filepath, dir) {
    return !(path_1.default.relative(filepath, dir) + path_1.default.sep).startsWith('..' + path_1.default.sep);
}
exports.pathIsInDirectory = pathIsInDirectory;
const stringToOptions = (str) => str
    ? Object.fromEntries(str.split(',').map(s => [s.replace(/^no-/, ''), !s.startsWith('no-')]))
    : {};
exports.stringToOptions = stringToOptions;
function addScriptToHtmlHead(html, source) {
    const htmlRoot = (0, node_html_parser_1.parse)(html);
    const scriptNode = new node_html_parser_1.HTMLElement('script', {}, typeof source === 'string' ? `src=${JSON.stringify(source)}` : '', null);
    if (source instanceof Object) {
        scriptNode.textContent =
            'script' in source
                ? source.script
                : Object.entries(source)
                    .map(([k, v]) => `const ${k} = ${JSON.stringify(v)};`)
                    .join('\n');
    }
    if (true) {
        return html.replace(/(<\/head>)/, '$1' + scriptNode.outerHTML);
    }
    if (!htmlRoot.querySelector('head')) {
        htmlRoot.querySelector('body').appendChild(new node_html_parser_1.HTMLElement('head', {}, '', null));
    }
    htmlRoot.querySelector('head').appendChild(scriptNode);
    return htmlRoot.outerHTML;
}
exports.addScriptToHtmlHead = addScriptToHtmlHead;
